import copy
from datetime import timedelta, datetime
from typing import List, Dict
from . import HeatingMode


class TimeProgramDaySetting:
    """
    This class represents a time program setting within a day

    Args:
        start_time: Start time of the setting (format hh:mm)
        target_temperature: Target temperature of the setting
        mode: The mode that will be applied to the component (always None for a :class: `vr900connector.Room`)
    """
    def __init__(self, start_time: str, target_temperature: float, mode: HeatingMode):
        self.start_time = start_time
        self.target_temperature = target_temperature
        self.mode = mode
        self.absolute_minutes = TimeProgramDaySetting.to_absolute_minute(start_time)

    @classmethod
    def to_absolute_minute(cls, start_time) -> int:
        split = start_time.split(":")
        if len(split) > 1:
            hour = int(split[0]) * 60
            minute = int(split[1])
            return hour + minute
        raise ValueError(start_time)

    def __deepcopy__(self, memodict={}):
        return TimeProgramDaySetting(self.start_time, self.target_temperature, self.mode)


class TimeProgramDay:
    """
    This class represents a time program day, it's basically a list of :class: `vr900connector.TimeProgramDaySetting`

    Args:
        time_program_day_settings: list of settings (:class: `vr900connector.TimeProgramDaySetting`)
    """

    def __init__(self, time_program_day_settings: List[TimeProgramDaySetting]):
        self.time_program_day_settings = time_program_day_settings


class TimeProgram:
    """
    This class represents a time program (a week)

    Args:
        time_program_days: List of time program day (:class: `vr900connector.TimeProgramDay`)
    """

    def __init__(self, time_program_days: Dict[str, TimeProgramDay]):
        self.time_program_days = time_program_days

    def get_time_program_for(self, search_date: datetime) -> TimeProgramDaySetting:
        """
        This return the corresponding time program day setting for a given date

        :param search_date: The date for which you want to get the :class:`vr900connector.TimeProgramDaySetting`
        :return: The time program day setting corresponding to the date
        """
        day = search_date.strftime("%A").lower()
        day_before = (search_date - timedelta(days=1)).strftime("%A").lower()
        time = str(search_date.hour) + ':' + str(search_date.minute)

        absolute_minute = TimeProgramDaySetting.to_absolute_minute(time)
        timeProgramDay = self.time_program_days[day]
        timeProgramDayBefore = self.time_program_days[day_before]

        # if given hour:minute is before the first setting of the day, get last setting of the previous day
        if absolute_minute < timeProgramDay.time_program_day_settings[0].absolute_minutes:
            return copy.deepcopy(timeProgramDayBefore.time_program_day_settings[-1])
        else:
            idx = 0
            while idx < len(timeProgramDay.time_program_day_settings) - 1:
                if absolute_minute > timeProgramDay.time_program_day_settings[idx].absolute_minutes and \
                        (idx + 1 == len(timeProgramDay.time_program_day_settings)
                         or absolute_minute < timeProgramDay.time_program_day_settings[idx + 1].absolute_minutes):
                    return copy.deepcopy(timeProgramDay.time_program_day_settings[idx])
                idx += 1

            # if no match a this point, it means search date is after the last setting of the day
            return copy.deepcopy(timeProgramDay.time_program_day_settings[-1])



class Device:
    """
    This class represents a device in a room

    Args:
        name: Name of the device
        sgtin: Unique identifier of a device
        device_type: Device type (I only know 'VALVE' for now)
        battery_low: is battery device low
        radio_out_of_reach: is device out of reach
    """

    def __init__(self, name: str, sgtin: str, device_type: str, battery_low: bool, radio_out_of_reach: bool):
        self.name = name
        self.sgtin = sgtin
        self.device_type = device_type
        self.battery_low = battery_low
        self.radio_out_of_reach = radio_out_of_reach






import logging
from datetime import date, timedelta

from .api import ApiConnector, Urls, Payloads, Defaults
from .model import Mapper, System, HotWater, QuickMode, QuickVeto, Room, Zone, HeatingMode, Circulation, Constants

LOGGER = logging.getLogger('SystemManager')


class SystemManager:
    """
    This is the main class to use if you want to do more advanced things with your system.

    The manager is throwing :exc:`vr900connector.api.ApiError` (thrown by :class:`vr900connector.api.ApiConnector`)
    without altering it

    Args:
        user: User for login
        password: Password for login
        smart_phone_id: Smart phone id required by the API
        file_path: Where to store files created by the underlying connector.
    """

    def __init__(self, user: str, password: str, smart_phone_id: str = Defaults.SMART_PHONE_ID,
                 file_path: str = Defaults.FILES_PATH):
        self._connector = ApiConnector(user, password, smart_phone_id, file_path)

    def get_system(self):
        full_system = self._connector.get(Urls.system())
        live_report = self._connector.get(Urls.live_report())
        hvac_state = self._connector.get(Urls.hvac())

        holiday_mode = Mapper.holiday_mode(full_system)
        boiler_status = Mapper.boiler_status(hvac_state, live_report)

        zones = Mapper.zones(full_system)

        rooms = None
        for zone in zones:
            if zone.rbr:
                raw_rooms = self._connector.get(Urls.rooms())
                rooms = Mapper.rooms(raw_rooms)
                break

        hot_water = Mapper.domestic_hot_water(full_system, live_report)
        circulation = Mapper.circulation(full_system)

        outdoorTemperature = Mapper.outdoor_temp(full_system)
        quickMode = Mapper.quick_mode(full_system)

        return System(holiday_mode, boiler_status, zones, rooms, hot_water, circulation, outdoorTemperature, quickMode)

    def get_hot_water(self, hot_water: HotWater):
        full_system = self._connector.get(Urls.hot_water(hot_water.id))
        live_report = self._connector.get(Urls.live_report())
        return Mapper.domestic_hot_water_alone(full_system, hot_water.id, live_report)

    def get_room(self, room: Room):
        new_room = self._connector.get(Urls.room(room.id))
        return Mapper.room(new_room)

    def get_zone(self, zone: Zone):
        new_zone = self._connector.get(Urls.zone(zone.id))
        return Mapper.zone(new_zone)

    def get_circulation(self, circulation: Circulation):
        new_circulation = self._connector.get(Urls.circulation(circulation.id))
        return Mapper.circulation_alone(new_circulation, circulation.id)

    def set_hot_water_setpoint_temperature(self, hot_water: HotWater, temperature: float):
        """
        This set the target temperature for the hotwater.

        :param hot_water: the hot_water you want to set target temperature
        :param temperature: the temperature
        :return: True/False whether the update occurred or not
        """
        LOGGER.info("Will try to set dhw target temperature to %s", temperature)
        if temperature and hot_water:
            self._connector.put(Urls.hot_water_temperature_setpoint(hot_water.id),
                                Payloads.hotwater_temperature_setpoint(self._round(temperature)))
            return True
        else:
            LOGGER.debug("No temperature nor hot_water provided, nothing to do")
            return False

    def set_hot_water_operation_mode(self, hotwater: HotWater, new_mode: HeatingMode):
        """
        Set new operation mode for the hot water.

        :param hotwater: the :class:`vr900connector.model.HotWater` representing to current hotwater component
        :param new_mode: Name of the new mode, see :mod:`vr900connector.model.HeatingMode`
        :return: True/False whether new_mode has been set or not
        """
        LOGGER.info("Will try to set hot water mode to %s", new_mode)

        if hotwater:
            if new_mode:
                if new_mode in HotWater.MODES:
                    LOGGER.debug("New mode is %s", new_mode)
                    self._connector.put(Urls.hot_water_operation_mode(hotwater.id),
                                        Payloads.hot_water_operation_mode(new_mode.name))
                    return True
                else:
                    LOGGER.debug("New mode is not available for hot water %s", new_mode)
                    return False
            else:
                LOGGER.debug("No new mode provided, nothing to do")
                return False
        else:
            LOGGER.debug("No hotwater provided")
            return False

    def set_room_operation_mode(self, room: Room, new_mode: HeatingMode):
        """
        Set new operation mode for a room.

        :param room: the :class:`vr900connector.model.Room` representing to current room component
        :param new_mode:
        :return: True/False whether new_mode has been set or not
        """

        if room:
            if new_mode:
                if new_mode in Room.MODES and new_mode != HeatingMode.QUICK_VETO:
                    LOGGER.debug("New mode is %s", new_mode)
                    self._connector.put(Urls.room_operation_mode(room.id), Payloads.room_operation_mode(new_mode.name))
                    return True
                else:
                    LOGGER.debug("New mode is not available for room %s", new_mode)
                    return False
            else:
                LOGGER.debug("No new mode provided, nothing to do")
                return False
        else:
            LOGGER.debug("No room provided")
            return False

    def set_zone_operation_mode(self, zone: Zone, new_mode: HeatingMode):
        """
        Set new operation mode for a zone.

        :param zone: the :class:`vr900connector.model.Zone` representing to current zone component
        :param new_mode:
        :return: True/False whether new_mode has been set or not
        """

        if zone:
            if new_mode:
                if new_mode in Zone.MODES and new_mode != HeatingMode.QUICK_VETO:
                    LOGGER.debug("New mode is %s", new_mode)
                    self._connector.put(Urls.zone_heating_mode(zone.id), Payloads.zone_operation_mode(new_mode.name))
                    return True
                else:
                    LOGGER.debug("New mode is not available for zone %s", new_mode)
                    return False
            else:
                LOGGER.debug("No new mode provided, nothing to do")
                return False
        else:
            LOGGER.debug("No zone provided")
            return False

    def set_quick_mode(self, current_quick_mode: QuickMode, new_quick_mode: QuickMode):
        """
        Set quick mode system wise
        :return: True/False whether new_mode has been set or not
        """

        if not current_quick_mode:
            if new_quick_mode:
                self._connector.put(Urls.system_quickmode(), Payloads.quickmode(new_quick_mode.name))
                return True
            else:
                LOGGER.debug("No new quick mode provided")
                return False
        else:
            LOGGER.debug("There is already a quick mode in place: %s", current_quick_mode.name)
            return False

    def set_room_quick_veto(self, room: Room, quick_veto: QuickVeto):
        if quick_veto and room:
            self._connector.put(Urls.room_quick_veto(room.id),
                                Payloads.room_quick_veto(quick_veto.target_temperature, quick_veto.remaining_time))
            return True
        else:
            LOGGER.debug("Quick veto %s or room %s not provided", quick_veto, room)
            return False

    def remove_room_quick_veto(self, room: Room):
        self._connector.delete(Urls.room_quick_veto(room.id))
        return True

    def set_zone_quick_veto(self, zone: Zone, quick_veto: QuickVeto):
        if quick_veto and zone:
            self._connector.put(Urls.zone_quick_veto(zone.id),
                                Payloads.zone_quick_veto(quick_veto.target_temperature))
            return True
        else:
            LOGGER.debug("Quick veto %s or zone %s not provided", quick_veto, zone)
            return False

    def remove_zone_quick_veto(self, zone: Zone):
        self._connector.delete(Urls.zone_quick_veto(zone.id))
        return True

    def set_room_setpoint_temperature(self, room: Room, temperature: float):
        """
        This set the target temperature for a room.

        :param room: the room you want to set target temperature
        :param temperature: the temperature
        :return: True/False whether the update occurred or not
        """
        LOGGER.info("Will try to set room target temperature to %s", temperature)
        if temperature and room:
            self._connector.put(Urls.room_set_temperature_setpoint(room.id),
                                Payloads.room_temperature_setpoint(self._round(temperature)))
            return True
        else:
            LOGGER.debug("No temperature nor room provided, nothing to do")
            return False

    def set_zone_setpoint_temperature(self, zone: Zone, temperature: float):
        """
        This set the target temperature for a zone.

        :param zone: the zone you want to set target temperature
        :param temperature: the temperature
        :return: True/False whether the update occurred or not
        """
        LOGGER.info("Will try to set zone target temperature to %s", temperature)
        if temperature and zone:
            self._connector.put(Urls.zone_heating_setpoint_temperature(zone.id),
                                Payloads.zone_temperature_setpoint(self._round(temperature)))
            return True
        else:
            LOGGER.debug("No temperature nor zone provided, nothing to do")
            return False

    def set_zone_setback_temperature(self, zone: Zone, temperature: float):
        """
        This set the setback temperature for a zone.

        :param zone: the zone you want to set setback temperature
        :param temperature: the temperature
        :return: True/False whether the update occurred or not
        """
        LOGGER.info("Will try to set zone setback temperature to %s", temperature)
        if temperature and zone:
            self._connector.put(Urls.zone_heating_setback_temperature(zone.id),
                                Payloads.zone_temperature_setback(self._round(temperature)))
            return True
        else:
            LOGGER.debug("No temperature nor zone provided, nothing to do")
            return False

    def set_holiday_mode(self, start_date: date, end_date: date, temperature: float):
        """
        Set the holiday mode

        :param start_date: starting date of the holiday mode
        :param end_date: ending date of the holiday mode
        :param temperature: minimal temperature
        :return: True if update occurred
        """
        self._connector.put(Urls.system_holiday_mode(), Payloads.holiday_mode(True, start_date, end_date, temperature))
        return True

    def remove_holiday_mode(self, temperature: float = Constants.FROST_PROTECTION_TEMP):
        """
        Remove holiday mode. Set start date to two days before and end date to yesterday

        :param temperature: default is :class:`vr900connector.model.Constants#FROST_PROTECTION_TEMP`
        :return: True if update occurred
        """
        self._connector.put(Urls.system_holiday_mode(), Payloads.holiday_mode(False,
                                                                              date.today() - timedelta(days=2),
                                                                              date.today() - timedelta(days=1),
                                                                              temperature))
        return True

    def request_hvac_update(self):
        """
        Request an hvac update. Please note the update is done asynchronously. Most of the time, it takes about 1 or 2
        minutes before you can see the new status in :class:`vr900connector.model.BoilerStatus`
        :return: True if the update request was accepted by the API
        """
        self._connector.put(Urls.hvac_update())
        return True

    def logout(self):
        """
        Get logged out from the API
        """
        self._connector.logout()

    def _round(self, number: float):
        """
        This function round a float to the nearest 0.5, as vaillant API only accepts 0.5 step
        :param number: the number to round
        :return: the rounded number
        """
        return round(number * 2) / 2



from datetime import datetime
import abc

from . import TimeProgram, QuickVeto, ActiveMode, HeatingMode


class Component:
    """
    This class represents a component in your installation (room, zone, hot water, circulation).

    Args:
        component_id: id of the component
        name: name of the component (this is the name you defined in the application or on VRC700)
        time_program: The timeprogram you defined, see :class:`vr900connector.model.TimeProgram`
        current_temperature: The current temperature of the component
        target_temperature: The target temperature of the component
        operation_mode: Configured operation mode for the component. It can be completely different from the
        :class:`vr900connector.model.ActiveMode`. To get the real running mode, consider using
        :func:`vr900connector.model.System.get_active_mode_zone()`
        quick_veto: it there is a quick veto running on
    """

    def __init__(self, component_id: any, name: str, time_program: TimeProgram, current_temperature: float,
                 target_temperature: float, operation_mode: HeatingMode, quick_veto: QuickVeto):
        self.id = component_id
        self.name = name
        self.time_program = time_program
        self.current_temperature = current_temperature
        self.target_temperature = target_temperature
        self.operation_mode = operation_mode
        self.quick_veto = quick_veto

    @property
    def active_mode(self) -> ActiveMode:
        """
        Get the active mode for a component. Please note that a component is not aware of quick mode or holiday mode

        :return: ActiveMode
        """

        if self.quick_veto:
            return ActiveMode(self.quick_veto.target_temperature, HeatingMode.QUICK_VETO)

        if self.operation_mode == HeatingMode.AUTO:
            setting = self.time_program.get_time_program_for(datetime.now())
            if setting.target_temperature:
                return ActiveMode(setting.target_temperature, HeatingMode.AUTO, setting.mode)
            else:
                return self._get_specific_active_mode()
        else:
            return self._get_specific_active_mode()

    @abc.abstractmethod
    def _get_specific_active_mode(self) -> ActiveMode:
        """
        Get active mode for specific mode other than 'AUTO'

        :return: ActiveMode
        """



import logging
import os
import pickle
from pathlib import Path

_LOGGER = logging.getLogger('FileUtils')


class FileUtils:

    @classmethod
    def load_from_file(cls, path: str, filename: str):
        join_path = cls._join_path(path, filename)
        try:
            with join_path.open("rb") as f:
                return pickle.load(f)

        except Exception:
            _LOGGER.debug("Cannot open file: %s", join_path, exc_info=True)
            return None

    @classmethod
    def save_to_file(cls, data: any, path: str, filename:str):
        join_path = cls._join_path(path, filename)
        _LOGGER.debug("Will save data to %s", join_path)
        try:

            with join_path.open("wb+") as f:
                pickle.dump(data, f)

        except Exception:
            _LOGGER.debug("Cannot save data: %s to %s", str(data), join_path, exc_info=True)

    @classmethod
    def delete_file(cls, path: str, filename: str):
        join_path = cls._join_path(path, filename)
        try:
            os.remove(str(join_path))
        except Exception:
            _LOGGER.debug("Cannot delete file %s", join_path, exc_info=True)

    @classmethod
    def delete_dir(cls, path: str):
        try:
            os.rmdir(path)
        except Exception as e:
            _LOGGER.debug("Cannot delete dir %s", path, exc_info=True)

    @classmethod
    def _join_path(cls, path: str, filename: str):
        file_path = Path(path)
        file_path.mkdir(exist_ok=True, parents=True)
        return file_path.joinpath(filename)



from datetime import date

from vr900connector.model import ActiveMode, QuickMode


class HolidayMode:
    """
    Represents system's holiday mode

    Args:
        active: If holiday mode is active
        start_date: Start date of the holiday mode
        end_date: End date of the holiday mode
        target_temperature: Target temperature during holiday mode
    """

    def __init__(self, active: bool, start_date: date, end_date: date, target_temperature: float):
        self.active = active
        self.start_date = start_date
        self.end_date = end_date
        self.target_temperature = target_temperature

    @property
    def active_mode(self) -> ActiveMode:
        if self.active:
            return ActiveMode(self.target_temperature, QuickMode.HOLIDAY)



from datetime import date


class Payloads:

    _DATE_FORMAT = "%Y-%m-%d"

    """
    This is meant to be used with :mod: `vr900connector.api.urls` to allow user to easily obtain the payload for a
    request.
    Payload are always json formatted
    """

    @classmethod
    def hotwater_temperature_setpoint(cls, temperature: float):
        """
        Payload used to set target temperature for *hotwater*
        """
        return {"temperature_setpoint": temperature}

    @classmethod
    def room_temperature_setpoint(cls, temperature: float):
        """
        Payload used to set target temperature for *room*
        """
        return {
            "temperatureSetpoint": temperature
        }

    @classmethod
    def zone_temperature_setpoint(cls, temperature: float):
        """
        Payload used to set target temperature for *zone*
        """
        return {
            "setpoint_temperature": temperature
        }

    @classmethod
    def zone_temperature_setback(cls, temperature: float):
        """
        Payload used to set setback temperature for *zone*
        """
        return {
            "setback_temperature": temperature
        }

    @classmethod
    def hot_water_operation_mode(cls, mode: str):
        """
        Payload to set operation mode for *hotwater*
        """
        return {"operation_mode": mode}

    @classmethod
    def room_operation_mode(cls, mode: str):
        """
        Payload to set operation mode for *room*
        """
        return {"operationMode": mode}

    @classmethod
    def zone_operation_mode(cls, mode: str):
        """
        Payload to set operation mode for *zone*
        """
        return {"mode": mode}

    @classmethod
    def quickmode(cls, quick_mode: str, duration: int = None):
        """
        Payload to set quick mode for the system.
        Duration is mandatory (Duration is in minutes, max 1440 =24 hours)
        """
        payload = {
            "quickmode":
                {
                    "quickmode": quick_mode,
                }
        }

        if duration:
            payload["quickmode"]["duration"] = duration

        return payload

    @classmethod
    def zone_quick_veto(cls, temperature: float):
        """
        Payload to set a quick veto for a *Zone*.
        The duration is not configurable by the API, it's 6 hours
        """
        return {
            "setpoint_temperature": temperature
        }

    @classmethod
    def room_quick_veto(cls, temperature: float, duration: int):
        """
        Payload to set a quick veto for a *Room*.
        Duration is mandatory (Duration is in minutes, max 1440 =24 hours)
        """
        return {
            "temperatureSetpoint": temperature,
            "duration": duration
        }

    @classmethod
    def holiday_mode(cls, active: bool, start_date: date, end_date: date, temperature: float):
        """
        Payload to set holiday mode
        """
        return {
            "active": active,
            "start_date": start_date.strftime(cls._DATE_FORMAT),
            "end_date": end_date.strftime(cls._DATE_FORMAT),
            "temperature_setpoint": temperature
        }



from datetime import datetime

from . import Component, TimeProgram, QuickVeto, ActiveMode, Constants, HeatingMode


class Zone(Component):
    """
    This class represents a zone (configured with the VR700).

    If rbr (Room By Room) is True, the zone itself doesn't mean anything anymore, rooms are 'controlling' the zone.

    Args:
        target_min_temperature: Minimal temperature to reach
        active_function: Active function for the zone, basically, 'HEATING' or STAND BY'
        rbr: Set to True if the zone is controlled by rooms
    """

    MODES = [HeatingMode.AUTO, HeatingMode.OFF, HeatingMode.DAY, HeatingMode.NIGHT, HeatingMode.QUICK_VETO]
    """
    List of available modes for a zone
    """

    MIN_TEMP = Constants.FROST_PROTECTION_TEMP
    """
    Minimum temperature in celsius for a room, this is coming from documentation
    """

    MAX_TEMP = Constants.THERMOSTAT_MAX_TEMP
    """
    Maximum temperature celsius for a room, this is coming from my tests with android application, cannot go above 30
    """

    def __init__(self, component_id: any, name: str, time_program: TimeProgram, current_temperature: float,
                 target_temperature: float, operation_mode: HeatingMode, quick_veto: QuickVeto,
                 target_min_temperature: float, active_function: str, rbr: bool):
        super().__init__(component_id, name, time_program, current_temperature, target_temperature, operation_mode,
                         quick_veto)
        self.target_min_temperature = target_min_temperature
        self.active_function = active_function
        self.rbr = rbr

    def _get_specific_active_mode(self) -> ActiveMode:
        if self.operation_mode == HeatingMode.AUTO:
            setting = self.time_program.get_time_program_for(datetime.now())

            if setting.mode == HeatingMode.DAY:
                mode = ActiveMode(self.target_temperature, HeatingMode.AUTO, setting.mode)
            else:
                mode = ActiveMode(self.target_min_temperature, HeatingMode.AUTO, setting.mode)
        elif self.operation_mode == HeatingMode.OFF:
            mode = ActiveMode(self.MIN_TEMP, HeatingMode.OFF)
        elif self.operation_mode == HeatingMode.DAY:
            mode = ActiveMode(self.target_temperature, HeatingMode.DAY)
        else:  # MODE_NIGHT
            mode = ActiveMode(self.target_min_temperature, HeatingMode.NIGHT)

        return mode



class Urls:
    """
    Vaillant API Urls with placeholder when needed.
    All placeholders are resolved here except {serial_number} which is resolved by
    :class:`vr900connector.api.ApiConnector`
    """

    _BASE = 'https://smart.vaillant.com/mobile/api/v4'

    _BASE_AUTHENTICATE = _BASE + '/account/authentication/v1'
    _AUTHENTICATE = _BASE_AUTHENTICATE + '/authenticate'
    _NEW_TOKEN = _BASE_AUTHENTICATE + '/token/new'
    _LOGOUT = _BASE_AUTHENTICATE + '/logout'

    """Facility details"""
    _FACILITIES_LIST = _BASE + '/facilities'
    _FACILITIES = _FACILITIES_LIST + '/{serial_number}'
    _FACILITIES_DETAILS = _FACILITIES + '/system/v1/details'
    _FACILITIES_STATUS = _FACILITIES + '/system/v1/status'
    _FACILITIES_SETTINGS = _FACILITIES + '/storage'
    _FACILITIES_DEFAULT_SETTINGS = _FACILITIES + '/storage/default'
    _FACILITIES_INSTALLER_INFO = _FACILITIES + '/system/v1/installerinfo'

    """Rbr (Room by room)"""
    _RBR_BASE = _FACILITIES + '/rbr/v1'
    _RBR_INSTALLATION_STATUS = _RBR_BASE + '/installationStatus'
    _RBR_UNDERFLOOR_HEATING_STATUS = _RBR_BASE + '/underfloorHeatingStatus'

    """Rooms"""
    _ROOMS_LIST = _RBR_BASE + '/rooms'
    _ROOM = _ROOMS_LIST + '/{room_index}'
    _ROOM_CONFIGURATION = _ROOMS_LIST + '/{room_index}/configuration'
    _ROOM_QUICK_VETO = _ROOM_CONFIGURATION + '/quickVeto'
    _ROOM_TIMEPROGRAM = _ROOMS_LIST + '/{room_index}/timeprogram'
    _ROOM_OPERATION_MODE = _ROOM_CONFIGURATION + '/operationMode'
    _ROOM_CHILD_LOCK = _ROOM_CONFIGURATION + '/childLock'
    _ROOM_NAME = _ROOM_CONFIGURATION + '/name'
    _ROOM_DEVICE_NAME = _ROOM_CONFIGURATION + '/devices/{sgtin}/name'
    _ROOM_TEMPERATURE_SETPOINT = _ROOM_CONFIGURATION + '/temperatureSetpoint'

    """Repeaters"""
    _REPEATERS_LIST = _RBR_BASE + '/repeaters'
    _REPEATER_DELETE = _REPEATERS_LIST + '/{sgtin}'
    _REPEATER_SET_NAME = _REPEATERS_LIST + '/{sgtin}/name'

    """HVAC (heating, ventilation and Air-conditioning)"""
    _HVAC = _FACILITIES + '/hvacstate/v1/overview'
    _HVAC_REQUEST_UPDATE = _FACILITIES + '/hvacstate/v1/hvacMessages/update'

    """EMF (Embedded Metering Function)"""
    _LIVE_REPORT = _FACILITIES + '/livereport/v1'
    _LIVE_REPORT_DEVICE = _LIVE_REPORT + '/devices/{device_id}/reports/{report_id}'
    _PHOTOVOLTAICS_REPORT = _FACILITIES + '/spine/v1/currentPVMeteringInfo'
    _EMF_REPORT = _FACILITIES + '/emf/v1/devices'
    _EMF_REPORT_DEVICE = _EMF_REPORT + '/{device_id}'

    """System control"""
    _SYSTEM = _FACILITIES + '/systemcontrol/v1'
    _SYSTEM_CONFIGURATION = _SYSTEM + '/configuration'
    _SYSTEM_STATUS = _SYSTEM + '/status'
    _SYSTEM_DATETIME = _SYSTEM_STATUS + '/datetime'
    _SYSTEM_PARAMETERS = _SYSTEM + '/parameters'
    _SYSTEM_QUICK_MODE = _SYSTEM_CONFIGURATION + '/quickmode'
    _SYSTEM_HOLIDAY_MODE = _SYSTEM_CONFIGURATION + '/holidaymode'

    """DHW (Domestic Hot Water)"""
    _DHW = _SYSTEM + '/dhw/{dhw_id}'

    """Circulation"""
    _CIRCULATION = _DHW + '/circulation'
    _CIRCULATION_CONFIGURATION = _CIRCULATION + '/configuration'
    _CIRCULATION_TIMEPROGRAM = _CIRCULATION_CONFIGURATION + '/timeprogram'

    """Hot water"""
    _HOT_WATER = _DHW + '/hotwater'
    _HOT_WATER_CONFIGURATION = _HOT_WATER + '/configuration'
    _HOT_WATER_TIMEPROGRAM = _HOT_WATER_CONFIGURATION + '/timeprogram'
    _HOT_WATER_OPERATION_MODE = _HOT_WATER_CONFIGURATION + '/operation_mode'
    _HOT_WATER_TEMPERATURE_SETPOINT = _HOT_WATER_CONFIGURATION + '/temperature_setpoint'

    """Ventilation"""
    _VENTILATION = _SYSTEM + '/ventilation/{ventilation_id}'
    _VENTILATION_CONFIGURATION = _VENTILATION + '/fan/configuration'
    _VENTILATION_TIMEPROGRAM = _VENTILATION_CONFIGURATION + '/timeprogram'
    _VENTILATION_DAY_LEVEL = _VENTILATION_CONFIGURATION + '/day_level'
    _VENTILATION_NIGHT_LEVEL = _VENTILATION_CONFIGURATION + '/night_level'
    _VENTILATION_OPERATION_MODE = _VENTILATION_CONFIGURATION + '/operation_mode'

    """Zones"""
    _ZONES_LIST = _SYSTEM + '/zones'
    _ZONE = _ZONES_LIST + '/{zone_id}'
    _ZONE_CONFIGURATION = _ZONE + '/configuration'
    _ZONE_NAME = _ZONE_CONFIGURATION + '/name'
    _ZONE_QUICK_VETO = _ZONE_CONFIGURATION + '/quick_veto'

    """Zone heating"""
    _ZONE_HEATING_CONFIGURATION = _ZONE + '/heating/configuration'
    _ZONE_HEATING_TIMEPROGRAM = _ZONE + '/heating/timeprogram'
    _ZONE_HEATING_MODE = _ZONE_HEATING_CONFIGURATION + '/mode'
    _ZONE_HEATING_SETPOINT_TEMPERATURE = _ZONE_HEATING_CONFIGURATION + '/setpoint_temperature'
    _ZONE_HEATING_SETBACK_TEMPERATURE = _ZONE_HEATING_CONFIGURATION + '/setback_temperature'

    """Zone cooling"""
    _ZONE_COOLING_CONFIGURATION = _ZONE + '/cooling/configuration'
    _ZONE_COOLING_TIMEPROGRAM = _ZONE + '/cooling/timeprogram'
    _ZONE_COOLING_MODE = _ZONE_COOLING_CONFIGURATION + '/mode'
    _ZONE_COOLING_SETPOINT_TEMPERATURE = _ZONE_COOLING_CONFIGURATION + '/setpoint_temperature'
    _ZONE_COOLING_MANUAL_SETPOINT_TEMPERATURE = _ZONE_COOLING_CONFIGURATION + \
        '/manual_mode_cooling_temperature_setpoint'

    @classmethod
    def new_token(cls) -> str:
        """
        Url to request a new token.
        """
        return Urls._NEW_TOKEN

    @classmethod
    def authenticate(cls) -> str:
        """
        Url to authenticate the user and receive cookies.
        """
        return Urls._AUTHENTICATE

    @classmethod
    def logout(cls) -> str:
        """
        Url to logout from the application, cookies are invalidated.
        """
        return Urls._LOGOUT

    @classmethod
    def facilities_list(cls) -> str:
        """
        Url to get the list of serial numbers of the facilities (and some other properties). For now, the connector only
        handle one serial number.
        """
        return Urls._FACILITIES_LIST

    @classmethod
    def rbr_underfloor_heating_status(cls) -> str:
        """
        Url to check if underfloor heating is installed or not
        """
        return Urls._RBR_UNDERFLOOR_HEATING_STATUS.format(serial_number='{serial_number}')

    @classmethod
    def rbr_installation_status(cls) -> str:
        """
        Url to check installation status
        """
        return Urls._RBR_INSTALLATION_STATUS.format(serial_number='{serial_number}')

    @classmethod
    def rooms(cls) -> str:
        """
        Url to get the list of rooms
        """
        return Urls._ROOMS_LIST.format(serial_number='{serial_number}')

    @classmethod
    def room(cls, room_index: int) -> str:
        """
        Url to get specific room details (configuration, timeprogram)
        """
        return Urls._ROOM.format(serial_number='{serial_number}', room_index=room_index)

    @classmethod
    def room_configuration(cls, room_index: int) -> str:
        """
        Url to get configuration for a room (name, temperature, target temperature, etc.)
        """
        return Urls._ROOM_CONFIGURATION.format(serial_number='{serial_number}', room_index=room_index)

    @classmethod
    def room_quick_veto(cls, room_index: int) -> str:
        """
        Url to get configuration for a room (name, temperature, target temperature, etc.)
        """
        return Urls._ROOM_QUICK_VETO.format(serial_number='{serial_number}', room_index=room_index)

    @classmethod
    def room_operation_mode(cls, room_index: int) -> str:
        """
        Url to set operation for a room
        """
        return Urls._ROOM_OPERATION_MODE.format(serial_number='{serial_number}', room_index=room_index)

    @classmethod
    def room_timeprogram(cls, room_index: int) -> str:
        """
        Url to get configuration for a room (name, temperature, target temperature, etc.)
        """
        return Urls._ROOM_TIMEPROGRAM.format(serial_number='{serial_number}', room_index=room_index)

    @classmethod
    def room_set_child_lock(cls, room_index: int) -> str:
        """
        Url to set child lock for all devices in a room.
        """
        return Urls._ROOM_CHILD_LOCK.format(serial_number='{serial_number}', room_index=room_index)

    @classmethod
    def room_set_name(cls, room_index: int) -> str:
        """
        Url to set child lock for all devices in a room.
        """
        return Urls._ROOM_NAME.format(serial_number='{serial_number}', room_index=room_index)

    @classmethod
    def room_set_device_name(cls, room_index: int, sgtin) -> str:
        """
        Url to set child lock for all devices in a room.
        """
        return Urls._ROOM_DEVICE_NAME.format(serial_number='{serial_number}', room_index=room_index, sgtin=sgtin)

    @classmethod
    def room_set_temperature_setpoint(cls, room_index: int) -> str:
        """
        Url to set child lock for all devices in a room.
        """
        return Urls._ROOM_TEMPERATURE_SETPOINT.format(serial_number='{serial_number}', room_index=room_index)

    @classmethod
    def repeaters(cls) -> str:
        """
        Url to get list of repeaters
        """
        return Urls._REPEATERS_LIST.format(serial_number='{serial_number}')

    @classmethod
    def delete_repeater(cls, sgtin) -> str:
        """
        Url to delete a repeater
        """
        return Urls._REPEATER_DELETE.format(serial_number='{serial_number}', sgtin=sgtin)

    @classmethod
    def set_repeater_name(cls, sgtin) -> str:
        """
        Url to set repeater's name
        """
        return Urls._REPEATER_SET_NAME.format(serial_number='{serial_number}', sgtin=sgtin)

    @classmethod
    def hvac(cls) -> str:
        """
        Url of the hvac overview
        """
        return Urls._HVAC.format(serial_number='{serial_number}')

    @classmethod
    def hvac_update(cls) -> str:
        """
        Url to request hvac update
        """
        return Urls._HVAC_REQUEST_UPDATE.format(serial_number='{serial_number}')

    @classmethod
    def live_report(cls) -> str:
        """
        Url to get live report data (current boiler water temperature, current hot water temperature, etc.)
        """
        return Urls._LIVE_REPORT.format(serial_number='{serial_number}')

    @classmethod
    def live_report_device(cls, device_id, report_id) -> str:
        """
        Url to get live report for specific device
        """
        return Urls._LIVE_REPORT_DEVICE.format(serial_number='{serial_number}', device_id=device_id,
                                               report_id=report_id)

    @classmethod
    def photovoltaics(cls) -> str:
        """
        Url to get photovoltaics data
        """
        return Urls._PHOTOVOLTAICS_REPORT.format(serial_number='{serial_number}')

    @classmethod
    def emf_report(cls) -> str:
        """
        Url to get emf (Embedded Metering Function) report
        """
        return Urls._EMF_REPORT.format(serial_number='{serial_number}')

    @classmethod
    def emf_report_device(cls, device_id) -> str:
        """
        Url to get emf (Embedded Metering Function) report for a specific device
        """
        return Urls._EMF_REPORT_DEVICE.format(serial_number='{serial_number}', device_id=device_id)

    @classmethod
    def facilities_details(cls) -> str:
        """
        Url to get facility detail
        """
        return Urls._FACILITIES_DETAILS.format(serial_number='{serial_number}')

    @classmethod
    def facilities_status(cls) -> str:
        """
        Url to get facility status
        """
        return Urls._FACILITIES_STATUS.format(serial_number='{serial_number}')

    @classmethod
    def facilities_settings(cls) -> str:
        """
        Url to get facility settings
        """
        return Urls._FACILITIES_SETTINGS.format(serial_number='{serial_number}')

    @classmethod
    def facilities_default_settings(cls) -> str:
        """
        Url to get facility default settings
        """
        return Urls._FACILITIES_DEFAULT_SETTINGS.format(serial_number='{serial_number}')

    @classmethod
    def facilities_installer_info(cls) -> str:
        """
        Url to get facility default settings
        """
        return Urls._FACILITIES_INSTALLER_INFO.format(serial_number='{serial_number}')

    @classmethod
    def system(cls) -> str:
        """
        Url to get full system (zones, dhw, ventilation, holiday mode, etc.) except rooms
        """
        return Urls._SYSTEM.format(serial_number='{serial_number}')

    @classmethod
    def system_configuration(cls) -> str:
        """
        Url to get system configuration (holiday mode, quick mode etc.)
        """
        return Urls._SYSTEM_CONFIGURATION.format(serial_number='{serial_number}')

    @classmethod
    def system_status(cls) -> str:
        """
        Url to get outdoor temperature and datetime
        """
        return Urls._SYSTEM_STATUS.format(serial_number='{serial_number}')

    @classmethod
    def system_datetime(cls) -> str:
        """
        Url to set datetime
        """
        return Urls._SYSTEM_DATETIME.format(serial_number='{serial_number}')

    @classmethod
    def system_parameters(cls) -> str:
        """
        Url to get system control parameters
        """
        return Urls._SYSTEM_PARAMETERS.format(serial_number='{serial_number}')

    @classmethod
    def system_quickmode(cls) -> str:
        """
        Url to get system control quick mode
        """
        return Urls._SYSTEM_QUICK_MODE.format(serial_number='{serial_number}')

    @classmethod
    def system_holiday_mode(cls) -> str:
        """
        Url to get system control holiday mode
        """
        return Urls._SYSTEM_HOLIDAY_MODE.format(serial_number='{serial_number}')

    @classmethod
    def dhw(cls, dhw_id) -> str:
        """
        Url to get domestic hot water (hot water and circulation)
        """
        return Urls._DHW.format(serial_number='{serial_number}', dhw_id=dhw_id)

    @classmethod
    def circulation(cls, dhw_id) -> str:
        """
        Url to get circulation details
        """
        return Urls._CIRCULATION.format(serial_number='{serial_number}', dhw_id=dhw_id)

    @classmethod
    def circulation_configuration(cls, dhw_id) -> str:
        """
        Url to get circulation configuration
        """
        return Urls._CIRCULATION_CONFIGURATION.format(serial_number='{serial_number}', dhw_id=dhw_id)

    @classmethod
    def circulation_timeprogram(cls, dhw_id) -> str:
        """
        Url to get circulation timeprogram
        """
        return Urls._CIRCULATION_TIMEPROGRAM.format(serial_number='{serial_number}', dhw_id=dhw_id)

    @classmethod
    def hot_water(cls, dhw_id) -> str:
        """
        Url to get hot water detail
        """
        return Urls._HOT_WATER.format(serial_number='{serial_number}', dhw_id=dhw_id)

    @classmethod
    def hot_water_configuration(cls, dhw_id) -> str:
        """
        Url to get hot water configuration
        """
        return Urls._HOT_WATER_CONFIGURATION.format(serial_number='{serial_number}', dhw_id=dhw_id)

    @classmethod
    def hot_water_timeprogram(cls, dhw_id) -> str:
        """
        Url to get hot water timeprogram
        """
        return Urls._HOT_WATER_TIMEPROGRAM.format(serial_number='{serial_number}', dhw_id=dhw_id)

    @classmethod
    def hot_water_operation_mode(cls, dhw_id) -> str:
        """
        Url to set hot water operation mode, only if it's not a quick action
        """
        return Urls._HOT_WATER_OPERATION_MODE.format(serial_number='{serial_number}', dhw_id=dhw_id)

    @classmethod
    def hot_water_temperature_setpoint(cls, dhw_id) -> str:
        """
        Url to set hot water temperature setpoint
        """
        return Urls._HOT_WATER_TEMPERATURE_SETPOINT.format(serial_number='{serial_number}', dhw_id=dhw_id)

    @classmethod
    def ventilation(cls, ventilation_id) -> str:
        """
        Url to get ventilation details
        """
        return Urls._VENTILATION.format(serial_number='{serial_number}', ventilation_id=ventilation_id)

    @classmethod
    def ventilation_configuration(cls, ventilation_id) -> str:
        """
        Url to get ventilation configuration
        """
        return Urls._VENTILATION_CONFIGURATION.format(serial_number='{serial_number}', ventilation_id=ventilation_id)

    @classmethod
    def ventilation_timeprogram(cls, ventilation_id) -> str:
        """
        Url to get ventilation timeprogram
        """
        return Urls._VENTILATION_TIMEPROGRAM.format(serial_number='{serial_number}', ventilation_id=ventilation_id)

    @classmethod
    def set_ventilation_day_level(cls, ventilation_id) -> str:
        """
        Url to set ventilation day level
        """
        return Urls._VENTILATION_DAY_LEVEL.format(serial_number='{serial_number}', ventilation_id=ventilation_id)

    @classmethod
    def set_ventilation_night_level(cls, ventilation_id) -> str:
        """
        Url to set ventilation night level
        """
        return Urls._VENTILATION_NIGHT_LEVEL.format(serial_number='{serial_number}', ventilation_id=ventilation_id)

    @classmethod
    def set_ventilation_operation_mode(cls, ventilation_id) -> str:
        """
        Url to set ventilation operation mode
        """
        return Urls._VENTILATION_OPERATION_MODE.format(serial_number='{serial_number}', ventilation_id=ventilation_id)

    @classmethod
    def zones(cls) -> str:
        """
        Url to get zones
        """
        return Urls._ZONES_LIST.format(serial_number='{serial_number}')

    @classmethod
    def zone(cls, zone_id) -> str:
        """
        Url to get a specific zone
        """
        return Urls._ZONE.format(serial_number='{serial_number}', zone_id=zone_id)

    @classmethod
    def zone_configuration(cls, zone_id) -> str:
        """
        Url to get a specific zone configuration
        """
        return Urls._ZONE_CONFIGURATION.format(serial_number='{serial_number}', zone_id=zone_id)

    @classmethod
    def zone_name(cls, zone_id) -> str:
        """
        Url to set zone name
        """
        return Urls._ZONE_NAME.format(serial_number='{serial_number}', zone_id=zone_id)

    @classmethod
    def zone_quick_veto(cls, zone_id) -> str:
        """
        Url to get quick veto
        """
        return Urls._ZONE_QUICK_VETO.format(serial_number='{serial_number}', zone_id=zone_id)

    @classmethod
    def zone_heating_configuration(cls, zone_id) -> str:
        """
        Url to get zone heating configuration
        """
        return Urls._ZONE_HEATING_CONFIGURATION.format(serial_number='{serial_number}', zone_id=zone_id)

    @classmethod
    def zone_heating_timeprogram(cls, zone_id) -> str:
        """
        Url to get a zone heating timeprogram
        """
        return Urls._ZONE_HEATING_TIMEPROGRAM.format(serial_number='{serial_number}', zone_id=zone_id)

    @classmethod
    def zone_heating_mode(cls, zone_id) -> str:
        """
        Url to get a zone heating mode
        """
        return Urls._ZONE_HEATING_MODE.format(serial_number='{serial_number}', zone_id=zone_id)

    @classmethod
    def zone_heating_setpoint_temperature(cls, zone_id) -> str:
        """
        Url to set a zone setpoint temperature
        """
        return Urls._ZONE_HEATING_SETPOINT_TEMPERATURE.format(serial_number='{serial_number}', zone_id=zone_id)

    @classmethod
    def zone_heating_setback_temperature(cls, zone_id) -> str:
        """
        Url to set a zone setback temperature
        """
        return Urls._ZONE_HEATING_SETBACK_TEMPERATURE.format(serial_number='{serial_number}', zone_id=zone_id)

    @classmethod
    def zone_cooling_configuration(cls, zone_id) -> str:
        """
        Url to get a zone cooling configuration
        """
        return Urls._ZONE_COOLING_CONFIGURATION.format(serial_number='{serial_number}', zone_id=zone_id)

    @classmethod
    def zone_cooling_timeprogram(cls, zone_id) -> str:
        """
        Url to get zone cooling timeprogram
        """
        return Urls._ZONE_COOLING_TIMEPROGRAM.format(serial_number='{serial_number}', zone_id=zone_id)

    @classmethod
    def zone_cooling_mode(cls, zone_id) -> str:
        """
        Url to set a zone cooling mode
        """
        return Urls._ZONE_COOLING_MODE.format(serial_number='{serial_number}', zone_id=zone_id)

    @classmethod
    def zone_cooling_setpoint_temperature(cls, zone_id) -> str:
        """
        Url to set the cooling temperature setpoint
        """
        return Urls._ZONE_COOLING_SETPOINT_TEMPERATURE.format(serial_number='{serial_number}', zone_id=zone_id)

    @classmethod
    def zone_cooling_manual_setpoint_temperature(cls, zone_id) -> str:
        """
        Url to set manual cooling setpoint temperature
        """
        return Urls._ZONE_COOLING_MANUAL_SETPOINT_TEMPERATURE.format(serial_number='{serial_number}', zone_id=zone_id)



import datetime

from . import BoilerStatus, Circulation, Device, HolidayMode, HotWater, QuickMode, QuickVeto, Room, TimeProgram, \
    TimeProgramDay, TimeProgramDaySetting, Zone, HeatingMode

_DATE_FORMAT = "%Y-%m-%d"


class Mapper:

    @classmethod
    def quick_mode(cls, full_system):
        if full_system:
            quick_mode = full_system.get("body", dict()).get("configuration", dict()).get("quickmode")
            if quick_mode:
                qm = QuickMode[quick_mode.get("quickmode")]
                if qm != QuickMode.QM_QUICK_VETO:
                    return qm

    @classmethod
    def outdoor_temp(cls, full_system):
        if full_system:
            return full_system.get("body", dict()).get("status", dict()).get('outside_temperature')

    @classmethod
    def installation_name(cls, facilities):
        if facilities:
            return facilities.get("body", dict()).get("facilitiesList", list())[0].get("name")

    @classmethod
    def rooms(cls, raw_rooms):
        rooms = list()
        if raw_rooms:
            for raw_room in raw_rooms.get("body", dict()).get("rooms", list()):
                rooms.append(Mapper.room(raw_room))

        return rooms

    @classmethod
    def room(cls, raw_room):
        if raw_room:
            raw_room = raw_room.get("body", raw_room)

            if raw_room:
                config = raw_room.get("configuration", dict())

                component_id = raw_room.get("roomIndex")
                child_lock = config.get("childLock")
                target_temp = config.get("temperatureSetpoint")
                current_temp = config.get("currentTemperature")
                devices = Mapper.devices(config.get("devices"))
                window_open = config.get("isWindowOpen")
                name = config.get("name")
                operation_mode = HeatingMode[config.get("operationMode")]

                raw_quickVeto = config.get("quickVeto")
                quick_veto = None
                if raw_quickVeto:
                    quick_veto = QuickVeto(raw_quickVeto.get("remainingDuration"), config.get("temperatureSetpoint"))

                time_program = Mapper.time_program(raw_room.get("timeprogram"))

                return Room(component_id, name, time_program, current_temp, target_temp, operation_mode, quick_veto,
                            child_lock, window_open, devices)

    @classmethod
    def devices(cls, raw_devices):
        devices = list()
        if raw_devices:
            for raw_device in raw_devices:
                name = raw_device.get("name")
                device_type = raw_device.get("deviceType")
                battery_low = raw_device.get("isBatteryLow")
                radio_out_of_reach = raw_device.get("isRadioOutOfReach")
                sgtin = raw_device.get("sgtin")
                devices.append(Device(name, sgtin, device_type, battery_low, radio_out_of_reach))

        return devices

    @classmethod
    def time_program(cls, raw_time_program, mode_key_name=""):
        result = dict()
        if raw_time_program:
            result["monday"] = Mapper.time_program_day(raw_time_program.get("monday"), mode_key_name)
            result["tuesday"] = Mapper.time_program_day(raw_time_program.get("tuesday"), mode_key_name)
            result["wednesday"] = Mapper.time_program_day(raw_time_program.get("wednesday"), mode_key_name)
            result["thursday"] = Mapper.time_program_day(raw_time_program.get("thursday"), mode_key_name)
            result["friday"] = Mapper.time_program_day(raw_time_program.get("friday"), mode_key_name)
            result["saturday"] = Mapper.time_program_day(raw_time_program.get("saturday"), mode_key_name)
            result["sunday"] = Mapper.time_program_day(raw_time_program.get("sunday"), mode_key_name)

        return TimeProgram(result)

    @classmethod
    def time_program_day(cls, raw_time_program_day, mode_key_name=""):
        time_program_day_settings = list()
        if raw_time_program_day:
            for time_setting in raw_time_program_day:
                start_time = time_setting.get("startTime")
                target_temp = time_setting.get("temperatureSetpoint")
                mode = None
                if time_setting.get(mode_key_name):
                    mode = HeatingMode[time_setting.get(mode_key_name)]
                time_program_day_settings.append(TimeProgramDaySetting(start_time, target_temp, mode))

        return TimeProgramDay(time_program_day_settings)

    @classmethod
    def holiday_mode(cls, full_system):
        holidayMode = HolidayMode(False, None, None, None)
        if full_system:
            raw_holiday_mode = full_system.get("body", dict()).get("configuration", dict()).get("holidaymode")
            if raw_holiday_mode and raw_holiday_mode.get("active"):
                holidayMode.active = True
                holidayMode.targetTemperature = raw_holiday_mode.get("temperature_setpoint")
                holidayMode.startDate = datetime.datetime\
                    .strptime(raw_holiday_mode.get("start_date"), _DATE_FORMAT).date()
                holidayMode.endDate = datetime.datetime.strptime(raw_holiday_mode.get("end_date"), _DATE_FORMAT).date()

        return holidayMode

    @classmethod
    def boiler_status(cls, hvac_state, live_report):
        if hvac_state:
            hvac_state_info = Mapper._find_hvac_message_status(hvac_state)
            meta = hvac_state.get('meta', dict())
            if hvac_state_info:
                timestamp = hvac_state_info.get("timestamp")
                last_update = None
                if timestamp:
                    last_update = datetime.datetime.fromtimestamp(timestamp / 1000)

                device_name = hvac_state_info.get("deviceName")
                code = hvac_state_info.get("statusCode")
                title = hvac_state_info.get("title")
                description = hvac_state_info.get("description")
                hint = hvac_state_info.get("hint")
                online = meta.get('onlineStatus', dict()).get('status')
                update = meta.get('firmwareUpdateStatus', dict()).get('status')
                water_pressure = Mapper._find_water_pressure_report(live_report)
                boiler_temperature = Mapper._find_boiler_temperature_report(live_report)

                return BoilerStatus(device_name, description, title, code, hint, last_update, online, update,
                                    water_pressure, boiler_temperature)

    @classmethod
    def zones(cls, full_system):
        zones = list()
        if full_system:
            for raw_zone in full_system.get("body", dict()).get("zones", list()):
                zones.append(Mapper.zone(raw_zone))

        return zones

    @classmethod
    def zone(cls, raw_zone):
        raw_zone_body = raw_zone.get("body")

        if raw_zone_body:
            raw_zone = raw_zone_body

        if raw_zone:
            heating = raw_zone.get("heating", dict())
            configuration = raw_zone.get("configuration", dict())
            heating_configuration = heating.get("configuration", dict())

            zone_id = raw_zone.get("_id")
            operation_mode = HeatingMode[heating_configuration.get("mode")]
            target_temp = heating_configuration.get("setpoint_temperature")
            target_min_temp = heating_configuration.get("setback_temperature")
            time_program = Mapper.time_program(heating.get("timeprogram"), "setting")

            name = configuration.get("name", "").strip()
            current_temperature = configuration.get("inside_temperature")
            active_function = configuration.get("active_function")

            raw_quick_veto = configuration.get("quick_veto")
            quick_veto = None
            if raw_quick_veto and raw_quick_veto.get("active"):
                # No way to find start_date, Quick veto on zone lasts 6 hours
                quick_veto = QuickVeto(-1, raw_quick_veto.get("setpoint_temperature"))

            rbr = raw_zone.get("currently_controlled_by", dict()).get("name", "") == "RBR"

            return Zone(zone_id, name, time_program, current_temperature, target_temp, operation_mode, quick_veto,
                        target_min_temp, active_function, rbr)

    @classmethod
    def domestic_hot_water(cls, full_system, live_report):
        hot_water_list = None
        if full_system:
            hot_water_list = full_system.get("body", dict()).get("dhw")

        if hot_water_list:
            raw_hot_water = hot_water_list[0].get("hotwater")
            dwh_id = hot_water_list[0].get("_id")

            if raw_hot_water:
                return cls._map_hot_water(raw_hot_water, dwh_id, live_report)

    @classmethod
    def domestic_hot_water_alone(cls, raw_hot_water, dhw_id, live_report):
        if raw_hot_water:
            raw_hot_water_body = raw_hot_water.get("body", dict())
            return cls._map_hot_water(raw_hot_water_body, dhw_id, live_report)

    @classmethod
    def _map_hot_water(cls, raw_hot_water, dhw_id, live_report):
        target_temp = raw_hot_water.get("configuration", dict()).get("temperature_setpoint")
        raw_operation_mode = raw_hot_water.get("configuration", dict()).get("operation_mode")
        operation_mode = None
        if raw_operation_mode:
            operation_mode = HeatingMode[raw_operation_mode]
        time_program = Mapper.time_program(raw_hot_water.get("timeprogram", dict()), "mode")


        current_temp = None
        name = None
        if live_report:
            dhw_report = Mapper._find_dhw_temperature_report(live_report)

            if dhw_report:
                current_temp = dhw_report.get("value")
                name = dhw_report.get("name")

        return HotWater(dhw_id, name, time_program, current_temp, target_temp, operation_mode)

    @classmethod
    def circulation(cls, full_system):
        if full_system:
            hot_water_list = full_system.get("body", dict()).get("dhw", list())

            if hot_water_list:
                raw_circulation = hot_water_list[0].get("circulation")
                dhw_id = hot_water_list[0].get("_id")

                if raw_circulation:
                    return cls._map_circulation(raw_circulation, dhw_id)

    @classmethod
    def circulation_alone(cls, raw_circulation, dhw_id):
        if raw_circulation:
            raw_circulation_body = raw_circulation.get("body", dict())
            return cls._map_circulation(raw_circulation_body, dhw_id)

    @classmethod
    def _map_circulation(cls, raw_circulation, circulation_id):
        name = "Circulation"
        time_program = Mapper.time_program(raw_circulation.get("timeprogram"), "setting")
        raw_operation_mode = raw_circulation.get("configuration", dict()).get("operationMode")
        operation_mode = None
        if raw_operation_mode:
            operation_mode = HeatingMode[raw_operation_mode]

        return Circulation(circulation_id, name, time_program, operation_mode)

    @classmethod
    def _find_hvac_message_status(cls, hvac_state):
        for message in hvac_state.get("body", dict()).get("errorMessages", list()):
            if message.get("type") == "STATUS":
                return message

    @classmethod
    def _find_water_pressure_report(cls, live_report):
        if live_report:
            for device in live_report.get("body", dict()).get("devices", list()):
                for report in device.get("reports", list()):
                    if report.get("associated_device_function") == "HEATING" \
                            and report.get("_id") == "WaterPressureSensor":
                        return report.get("value")

    @classmethod
    def _find_boiler_temperature_report(cls, live_report):
        if live_report:
            for device in live_report.get("body", dict()).get("devices", list()):
                for report in device.get("reports", list()):
                    if report.get("associated_device_function") == "HEATING" \
                            and report.get("_id") == "FlowTemperatureSensor":
                        return report.get("value")

    @classmethod
    def _find_dhw_temperature_report(cls, live_report):
        if live_report:
            for device in live_report.get("body", dict()).get("devices", list()):
                for report in device.get("reports", list()):
                    if report.get("associated_device_function") == "DHW" \
                            and report.get("_id") == "DomesticHotWaterTankTemperature":
                        return report



from enum import Enum


class Mode:
    pass


class QuickVeto(Mode):
    """
    This represents a quick veto which can be applied for a :class:`vr900connector.model.Room` or a
    :class:`vr900connector.model.Zone` only.

    For a room, quick veto duration is customizable and it's possible to get the remaining duration.

    For a zone, quick veto duration is NOT customizable and the API returns always '0' as remaining duration

    Args:
        remaining_time: The remaining time of the quick veto (in minute) or 0 if not available
        target_temperature: Target temperature of the quick veto
    """

    def __init__(self, remaining_time: int, target_temperature: float):
        if remaining_time > 1440:
            raise ValueError(remaining_time)

        self.remaining_time = remaining_time
        self.target_temperature = target_temperature


class HeatingMode(Mode, Enum):

    def __init__(self, mode_name: str):
        self._mode_name = mode_name

    AUTO = 'AUTO'
    ON = 'ON'
    OFF = 'OFF'
    MANUAL = 'MANUAL'
    DAY = 'DAY'
    NIGHT = 'NIGHT'
    QUICK_VETO = 'QUICK_VETO'


class QuickMode(Mode, Enum):
    """
    Quick modes are mode you can quickly set with the app. Most of the times, quick modes have system wise impact.
    Example: for 'ONE_DAY_AWAY', the quick mode will prevent zones and water heater from heating.

    This is different from quick veto, which will only impact one component.

    Quick mode are taking precedence in case of simultaneous quick veto and quick mode.

    This class is a helper to check what is impacted by a quick mode.

    Note that there is a default duration for quick modes (which can't be changed) but the API always returns '0' as
    remaining duration, so there is no way to know when quick mode will end

    Vaillant documentation link:
    http://elearning.vaillant.com/vrc700/ci/en/documents/uk/infopool/Operating_instructions.pdf

    Args:
        for_zone: does the quick mode apply to a zone ?
        for_room: does the quick mode apply to a room ?
        for_hot_water: does the quick mode apply to the hot_water ?
        for_circulation: does the quick mode apply to the circulation ?
    """

    def __init__(self, mode_name: str, for_zone: bool, for_room: bool, for_hot_water: bool, for_circulation: bool):
        self._mode_name = mode_name
        self.for_zone = for_zone
        self.for_room = for_room
        self.for_hot_water = for_hot_water
        self.for_circulation = for_circulation

    QM_HOTWATER_BOOST = ('QM_HOTWATER_BOOST', False, False, True, True)
    """
    The advanced function heats the water in the domestic hot water cylinder once until the desired DHW circuit 
    temperature set is reached or until you cancel the advanced function early. The heating installation will then 
    return to the pre-set mode
    """

    QM_VENTILATION_BOOST = ('QM_VENTILATION_BOOST', True, False, False, False)
    """
    This advanced function switches the zone off for 30 minutes. The frost protection function is activated, and hot 
    water  generation and circulation remain active. Ventilation is activated and works at the highest ventilation 
    level.
    The advanced function is automatically deactivated after 30 minutes or if you cancel the advanced function early. 
    The heating installation will then return to the pre-set mode
    """

    QM_ONE_DAY_AWAY = ('QM_ONE_DAY_AWAY', True, False, True, True)
    """
    Hot water generation and circulation are switched off and the frost protection is activated. The advanced function 
    is automatically deactivated after 24:00 hours or if you cancel the advanced function first. The heating 
    installation will then return to the pre-set mode. Ventilation is activated and works at the lowest ventilation 
    level.
    """

    QM_SYSTEM_OFF = ('QM_SYSTEM_OFF', True, True, True, True)
    """
    The heating function, hot water circuit and cooling are switched off. The frost protection function is activated.
    The circulation is switched off. Ventilation is activated and works at the lowest ventilation level.
    """

    QM_ONE_DAY_AT_HOME = ('QM_ONE_DAY_AT_HOME', True, False, False, False)
    """
    This advanced function activates Automatic mode for one day with the settings for Sunday, as set using the Time 
    programmes function. The advanced function is automatically deactivated after 24:00 hours or if you cancel the 
    advanced function first. The heating installation will then return to the pre-set mode.
    """

    QM_PARTY = ('QM_PARTY', True, False, False, False)
    """
    The advanced function brings the room temperature to the set desired Day temperature, in accordance with the set 
    time periods. The advanced function is deactivated after six hours or if you cancel it before the six hours is up. 
    The heating installation will then return to the pre-set mode.
    """

    HOLIDAY = ('HOLIDAY', True, True, True, True)
    """
    """

    QM_QUICK_VETO = ('QM_QUICK_VETO', False, False, False, False)
    """
    This advanced function activates a quick veto for one specific zone
    """

    @classmethod
    def for_zone(cls) -> list:
        sub_list = []

        for quickMode in QuickMode:
            if quickMode.for_zone:
                sub_list.append(quickMode)

        return sub_list

    @classmethod
    def for_room(cls) -> []:
        sub_list = []

        for quickMode in QuickMode:
            if quickMode.for_room:
                sub_list.append(quickMode)

        return sub_list

    @classmethod
    def for_hot_water(cls) -> []:
        sub_list = []

        for quickMode in QuickMode:
            if quickMode.for_hot_water:
                sub_list.append(quickMode)

        return sub_list

    @classmethod
    def for_circulation(cls) -> []:
        sub_list = []

        for quickMode in QuickMode:
            if quickMode.for_circulation:
                sub_list.append(quickMode)

        return sub_list



from datetime import datetime


class BoilerStatus:
    """
    Represents the status of the boiler

    Args:
        device_name: name of the boiler
        description: long description of the code
        title: short description of the code
        code: code representing the current status of the boiler (heating, off, error, etc.)
        hint: hint (provided by the API)
        last_update: last update of the status (provided by the API)
    """

    def __init__(self, device_name: str, description: str, title: str, code: str, hint: str, last_update: datetime,
                 online_status: str, update_status: str, water_pressure: float, current_temperature: float):
        self.device_name = device_name
        self.description = description
        self.title = title
        self.code = code
        self.hint = hint
        self.last_update = last_update
        self.online_status = online_status
        self.update_status = update_status
        self.water_pressure = water_pressure
        self.current_temperature = current_temperature

    @property
    def is_error(self) -> bool:
        return self.code and (self.code.startswith('F') or self.code == 'con')

    @property
    def is_online(self):
        return self.online_status == 'ONLINE'

    @property
    def is_up_to_date(self):
        return self.update_status == 'UPDATE_NOT_PENDING'




from requests import Response


class ApiError(Exception):
    """
    This exception is thrown when a communication error occurs with the vaillant API

    Attributes:
        message: A message describing the error
        response: The response returned by the API

    Args:
        message: A message describing the error
        response: The response returned by the API, if any
    """

    def __init__(self, message: str, response: Response, payload=None):
        self.message = message
        self.response = response
        self.payload = payload



from datetime import datetime

from . import Component, TimeProgram, ActiveMode, HeatingMode


class Circulation(Component):
    """
    Circulation is quite a special component since there is no temperature nor quick veto involved.
    This class only exists to have a clean hierarchy
    """

    MODES = [HeatingMode.ON, HeatingMode.OFF, HeatingMode.AUTO]
    """
    List of mode available for circulation
    """

    def __init__(self, component_id: any, name: str, time_program: TimeProgram, operation_mode: HeatingMode):
        super().__init__(component_id, name, time_program, None, None, operation_mode, None)

    def _get_specific_active_mode(self) -> ActiveMode:
        if self.operation_mode == HeatingMode.AUTO:
            setting = self.time_program.get_time_program_for(datetime.now())
            mode = ActiveMode(None, HeatingMode.AUTO, setting.mode)
        else:
            mode = ActiveMode(None, self.operation_mode)
        return mode



from datetime import datetime

from . import TimeProgram, Component, ActiveMode, HeatingMode


class HotWater(Component):
    """
    This class represents the hot water of your system
    """

    MODES = [HeatingMode.ON, HeatingMode.OFF, HeatingMode.AUTO]
    """
    List of modes available for hot water
    """

    MIN_TEMP = 35
    """
    Minimum temperature in celsius for the hot water, this is coming from documentation
    """

    MAX_TEMP = 70
    """
    Maximum temperature celsius for the hot water, this is coming from my tests with android application, 
    cannot go above 70
    """

    def __init__(self, component_id: any, name: str, time_program: TimeProgram, current_temperature: float,
                 target_temperature: float, operation_mode: HeatingMode):
        super().__init__(component_id, name, time_program, current_temperature, target_temperature, operation_mode,
                         None)

    def _get_specific_active_mode(self) -> ActiveMode:
        if self.operation_mode == HeatingMode.AUTO:
            setting = self.time_program.get_time_program_for(datetime.now())

            if setting.mode == HeatingMode.ON:
                mode = ActiveMode(self.target_temperature, HeatingMode.AUTO, setting.mode)
            else:
                mode = ActiveMode(HotWater.MIN_TEMP, HeatingMode.AUTO, setting.mode)

        elif self.operation_mode == HeatingMode.ON:
            mode = ActiveMode(self.target_temperature, HeatingMode.ON)
        else:  # MODE_OFF
            mode = ActiveMode(HotWater.MIN_TEMP, HeatingMode.OFF)

        return mode



import logging

import requests

from . import Urls, ApiError, Defaults
from ..util import FileUtils

_LOGGER = logging.getLogger('Connector')

_JSON_CONTENT_TYPE_HEADER = {'content-type': 'application/json'}


class ApiConnector:
    """
    This is the low level smart.vaillant.com API connector. This is returning the raw JSON from responses or
    :exc:`vr900connector.api.ApiError` if something goes wrong (basically, when response error code > 399).

    On the first call, the connector will login automatically. If login don't succeed the first time, it will try to
    login a second time (only if response error code is 401) before raising an :exc:`vr900connector.api.ApiError`

    On Other calls, if session is running, the connector will assume the session is ok. If response error code is 401,
    the connector will clear the session and retry to login. This implementation allows the connector to reconnect
    automatically when cookies are outdated.

    All GET against the API works, PUT/POST  are not yet implemented.

    Please use :mod:`vr900connector.api.urls` in order to generate URL to be passed to the connector.

    Args:
        user: User for login
        password: Password for login
        smart_phone_id: Smart phone id required by the API
        file_path: Where to store files created by this connector. Cookies and serial number are saved to avoid doing
        re-login between sessions
    """

    def __init__(self, user: str, password: str, smart_phone_id: str = Defaults.SMART_PHONE_ID,
                 file_path: str = Defaults.FILES_PATH):
        self._user = user
        self._password = password
        self._smart_phone_Id = smart_phone_id
        self._file_path = file_path
        self._serial_number = self._load_serial_number_from_file()
        self._session = self._create_or_load_session()

    def logout(self):
        """
        To get logged out of the API. It means, the connector will have to request a new token and ask for cookies.
        Files generated by the connector are also deleted
        """
        response = None
        try:
            response = self._session.request('POST', Urls.logout())
        except Exception as e:
            raise ApiError("Error during logout", response) from e
        finally:
            self._clear_session()

    def query(self, url: str, method: str = 'GET', payload=None):
        """
        Call the vaillant API url with the chosen method, please use :mod:`vr900connector.api.urls` in order to generate
        URL to be passed to the connector
        """

        return self._secure_call(method, url, payload)

    def get(self, url: str):
        """
        GET call to a vaillant API url please use :mod:`vr900connector.api.urls` in order to generate  URL to be passed
        to the connector
        """
        return self.query(url)

    def put(self, url: str, payload=None):
        """
        PUT call to a vaillant API url please use :mod:`vr900connector.api.urls` in order to generate  URL to be passed
        to the connector
        """
        return self.query(url, 'PUT', payload)

    def post(self, url: str, payload=None):
        """
        POST call to a vaillant API url please use :mod:`vr900connector.api.urls` in order to generate  URL to be passed
        to the connector.
        """
        return self.query(url, 'POST', payload)

    def delete(self, url: str):
        """
        DELETE call to a vaillant API url please use :mod:`vr900connector.api.urls` in order to generate  URL to be
        passed to the connector
        """
        return self.query(url, 'DELETE')

    def _secure_call(self, method: str, url: str, payload=None, re_login: bool = False):
        response = None
        safe_url = None
        try:
            self._login(re_login)

            safe_url = url.format(serial_number=self._serial_number)
            response = self._session.request(method,
                                             safe_url,
                                             json=payload,
                                             headers=None if payload is None else _JSON_CONTENT_TYPE_HEADER)

            if response.status_code > 399:
                if not re_login and response.status_code == 401:
                    _LOGGER.debug('Call to %s failed with HTTP 401, will try to re-login', safe_url)
                    return self._secure_call(method, url, payload, True)
                else:
                    raise ApiError('Received error from server url: ' + safe_url + ' and method ' + method,
                                   response, payload)
            if response.text:
                return response.json()
            else:
                return {"ok": "ok"}
        except ApiError:
            raise
        except Exception as e:
            raise ApiError('Cannot {} url: {}'.format(method, safe_url if safe_url else url), response, payload) from e

    def _login(self, force_login: bool = False):
        try:
            if force_login:
                self._clear_session()
            if not self._session.cookies:
                self._session = self._create_or_load_session()
                self._serial_number = self._load_serial_number_from_file()

                if not self._session.cookies:
                    _LOGGER.info(
                        'No previous session found, will try to logging with username: %s and smartphoneId: %s',
                        self._user, self._smart_phone_Id)

                    authtoken = self._request_token()
                    self._get_cookies(authtoken)

            if not self._serial_number:
                self._get_serial_number()
        except ApiError:
            raise
        except Exception as e:
            raise ApiError('Error during login', None) from e

    def _request_token(self):
        params = {
            "smartphoneId": self._smart_phone_Id,
            "username": self._user,
            "password": self._password
        }

        try:
            response = self._session.post(Urls.new_token(), json=params, headers=_JSON_CONTENT_TYPE_HEADER)
            if response.status_code == 200:
                _LOGGER.debug('Token generation successful')
                return response.json()['body']['authToken']
            else:
                raise ApiError('Authentication failed', response)

        except ApiError:
            raise
        except Exception as e:
            raise ApiError('Error during authentication', None) from e

    def _get_cookies(self, auth_token: str):
        params = {
            "smartphoneId": self._smart_phone_Id,
            "username": self._user,
            "authToken": auth_token
        }

        try:
            response = self._session.post(Urls.authenticate(), json=params, headers=_JSON_CONTENT_TYPE_HEADER)

            if response.status_code == 200:
                self._session.cookies = response.cookies
                _LOGGER.debug('Cookie successfully retrieved %s', self._session.cookies)
                self._save_cookies_to_file()
            else:
                raise ApiError('Cannot get cookies', response)
        except ApiError:
            raise
        except Exception as e:
            raise ApiError('Error while getting cookies', None) from e

    def _get_serial_number(self):
        try:
            response = self._session.get(Urls.facilities_list())

            if response.status_code == 200:
                _LOGGER.debug('Serial number successfully retrieved')
                self._serial_number = response.json()['body']['facilitiesList'][0]['serialNumber']
                self._save_serial_number_to_file()
            else:
                raise ApiError('Cannot get serial number', response)
        except ApiError:
            raise
        except Exception as e:
            raise ApiError('Cannot get serial number', None) from e

    def _create_or_load_session(self):
        session = requests.Session()
        cookies = self._load_cookies_from_file()
        _LOGGER.debug('Found cookies %s', cookies)
        if cookies is not None:
            session.cookies = cookies
        return session

    def _clear_session(self):
        self._clear_cookie()
        self._clear_serial_number()
        self._session.close()
        self._session = requests.session()
        FileUtils.delete_dir(self._file_path)

    def _save_cookies_to_file(self):
        FileUtils.save_to_file(self._session.cookies, self._file_path, Defaults.COOKIE_FILE_NAME)

    def _save_serial_number_to_file(self):
        FileUtils.save_to_file(self._serial_number, self._file_path, Defaults.SERIAL_NUMBER_FILE_NAME)

    def _load_cookies_from_file(self):
        return FileUtils.load_from_file(self._file_path, Defaults.COOKIE_FILE_NAME)

    def _load_serial_number_from_file(self):
        return FileUtils.load_from_file(self._file_path, Defaults.SERIAL_NUMBER_FILE_NAME)

    def _clear_cookie(self):
        FileUtils.delete_file(self._file_path, Defaults.COOKIE_FILE_NAME)

    def _clear_serial_number(self):
        FileUtils.delete_file(self._file_path, Defaults.SERIAL_NUMBER_FILE_NAME)
        self._serial_number = None



from typing import List

from . import Component, TimeProgram, QuickVeto, Device, ActiveMode, HeatingMode, Constants


class Room(Component):
    """
    This class represents a Room, which is typically a group of VR50/VR51

    Args:
        child_lock: Are all VR50 locked ? (It means you cannot change temperature using the VR50 directly)
        window_open: Set to True when the system detect temperature drop
        devices: List of devices in the room, see :class:`vr900connector.Device`
    """

    MODES = [HeatingMode.OFF, HeatingMode.MANUAL, HeatingMode.AUTO, HeatingMode.QUICK_VETO]
    """
    List of available modes for a room
    """

    MIN_TEMP = Constants.FROST_PROTECTION_TEMP
    """
    Minimum temperature in celsius for a room, this is coming from documentation
    """

    MAX_TEMP = Constants.THERMOSTAT_MAX_TEMP
    """
    Maximum temperature celsius for a room, this is coming from my tests with android application, cannot go above 30
    """

    def __init__(self, component_id: any, name: str, time_program: TimeProgram, current_temperature: float,
                 target_temperature: float, operation_mode: HeatingMode, quick_veto: QuickVeto, child_lock: bool,
                 window_open: bool, devices: List[Device]):
        super().__init__(component_id, name, time_program, current_temperature, target_temperature, operation_mode,
                         quick_veto)
        self.child_lock = child_lock
        self.window_open = window_open
        self.devices = devices

    def _get_specific_active_mode(self) -> ActiveMode:
        if self.operation_mode == HeatingMode.OFF:
            mode = ActiveMode(self.MIN_TEMP, HeatingMode.OFF)
        else:  # MODE_MANUAL
            mode = ActiveMode(self.target_temperature, HeatingMode.MANUAL)

        return mode



from . import Mode


class ActiveMode:
    """
    Represents the active mode for a :class:`vr900connector.model.Component`.

    Active mode for a component can ony be retrieve via :class:`vr900connector.SystemManager` since it can be quite
    complex (quick veto, quick mode, holiday mode, etc.)

    Args:
        target_temperature: The target temperature the component is trying to reach
        current_mode: Name of the current mode, list is available at :mod:`vr900connector.model.constants`
        sub_mode: Name of the sub mode (available when current_mode is 'AUTO' and active mode is not related to a Room),
        otherwise it will be None
    """

    def __init__(self, target_temperature: float, current_mode: Mode, sub_mode: Mode = None):
        self.target_temperature = target_temperature
        self.current_mode = current_mode
        self.sub_mode = sub_mode



import tempfile


class Defaults:
    """
    This contains some default parameters of the application
    """

    FILES_PATH = tempfile.gettempdir() + '/.vaillant_vr900_files'

    COOKIE_FILE_NAME = '.vr900-vaillant.cookies'
    SERIAL_NUMBER_FILE_NAME = '.vr900-vaillant.serial'

    SMART_PHONE_ID = 'vr900-connector'



import datetime
from typing import List

from . import ActiveMode, HolidayMode, HotWater, Room, Zone, BoilerStatus, Circulation, QuickMode, HeatingMode


class System:
    """
    This class represents the main class to manipulate vaillant system. This class is designed to know everything (or
    at least to know to who it has to delegate to know) about the system.

    If your are using :class:`vr900connector.SystemManager`, you should only use this class to interact with the system

    Args:
        holiday_mode: See :class:`vr900connector.HolidayMode`
        boiler_status: See :class:`vr900connector.BoilerStatus`
        zones: List of :class:`vr900connector.Zone` available in the system
        rooms: List of :class:`vr900connector.Room` available in the system
        hot_water: See :class:`vr900connector.HotWater`
        circulation: See :class:`vr900connector.Circulation`
        outdoor_temperature: Outdoor temperature, if available
        quick_mode: A :class:`vr900connector.QuickMode` if any is running on
    """

    def __init__(self, holiday_mode: HolidayMode, boiler_status: BoilerStatus, zones: List[Zone], rooms: List[Room],
                 hot_water: HotWater, circulation: Circulation, outdoor_temperature: float, quick_mode: QuickMode):
        if holiday_mode:
            self.holiday_mode = holiday_mode
        else:
            self.holiday_mode = HolidayMode(False, None, None, None)

        self.boiler_status = boiler_status

        if zones:
            self._zones_dict = dict((zone.id, zone) for zone in zones)
        else:
            self._zones_dict = dict()

        if rooms:
            self._rooms_dict = dict((room.id, room) for room in rooms)
        else:
            self._rooms_dict = dict()

        self.hot_water = hot_water
        self.circulation = circulation
        self.outdoor_temperature = outdoor_temperature
        self.quick_mode = quick_mode

    def get_zone(self, zone_id: str) -> Zone:
        return self._zones_dict[zone_id]

    def get_room(self, room_id: int) -> Room:
        return self._rooms_dict[int(room_id)]

    def set_zone(self, zone_id: str, zone: Zone):
        self._zones_dict[zone_id] = zone

    def set_room(self, room_id: int, room: Room):
        self._rooms_dict[int(room_id)] = room

    @property
    def rooms(self):
        return self._rooms_dict.values()

    @property
    def zones(self):
        return self._zones_dict.values()

    def get_active_mode_zone(self, zone: Zone) -> ActiveMode:
        # Holiday mode takes precedence over everything
        if self.holiday_mode.active:
            return self.holiday_mode.active_mode

        # Global system quick mode takes over zone settings
        if self.quick_mode and self.quick_mode.for_zone:
            if self.quick_mode == QuickMode.QM_VENTILATION_BOOST:
                return ActiveMode(Zone.MIN_TEMP, self.quick_mode)

            if self.quick_mode == QuickMode.QM_ONE_DAY_AWAY:
                return ActiveMode(zone.target_min_temperature, self.quick_mode)

            if self.quick_mode == QuickMode.QM_SYSTEM_OFF:
                return ActiveMode(Zone.MIN_TEMP, self.quick_mode)

            if self.quick_mode == QuickMode.QM_ONE_DAY_AT_HOME:
                today = datetime.datetime.now()
                sunday = today - datetime.timedelta(days=today.weekday() - 6)

                time_program = zone.time_program.get_time_program_for(sunday)
                return ActiveMode(time_program.target_temperature, self.quick_mode)

            if self.quick_mode == QuickMode.QM_PARTY:
                return ActiveMode(zone.target_temperature, self.quick_mode)

        return zone.active_mode

    def get_active_mode_room(self, room: Room) -> ActiveMode:
        # Holiday mode takes precedence over everything
        if self.holiday_mode.active:
            return self.holiday_mode.active_mode

        # Global system quick mode takes over room settings
        if self.quick_mode and self.quick_mode.for_room:
            if self.quick_mode == QuickMode.QM_SYSTEM_OFF:
                return ActiveMode(Room.MIN_TEMP, self.quick_mode)

        return room.active_mode

    def get_active_mode_circulation(self, circulation: Circulation = None) -> ActiveMode:
        if not circulation:
            circulation = self.circulation

        if self.holiday_mode.active:
            active_mode = self.holiday_mode.active_mode
            active_mode.target_temperature = None
            return active_mode

        if self.quick_mode and self.quick_mode.for_circulation:
            return ActiveMode(None, self.quick_mode)

        return circulation.active_mode

    def get_active_mode_hot_water(self, hot_water: HotWater = None) -> ActiveMode:
        if not hot_water:
            hot_water = self.hot_water

        if self.holiday_mode.active:
            active_mode = self.holiday_mode.active_mode
            active_mode.target_temperature = HotWater.MIN_TEMP
            return active_mode

        if self.quick_mode and self.quick_mode.for_hot_water:
            if self.quick_mode == QuickMode.QM_HOTWATER_BOOST:
                return ActiveMode(hot_water.target_temperature, self.quick_mode)

            if self.quick_mode == QuickMode.QM_SYSTEM_OFF:
                return ActiveMode(HotWater.MIN_TEMP, self.quick_mode)

            if self.quick_mode == QuickMode.QM_ONE_DAY_AWAY:
                return ActiveMode(HotWater.MIN_TEMP, self.quick_mode)

        return hot_water.active_mode



import json
import inspect

from vr900connector.api import ApiConnector, ApiError, Defaults, Urls

import argparse

_IGNORE_LIST = ['new_token', 'authenticate', 'logout']


def main():
    parser = argparse.ArgumentParser(
        description='Read states of vaillant API')
    parser.add_argument(
        'username',
        help='username')
    parser.add_argument(
        'password',
        help='password')
    parser.add_argument(
        '-fp', '--file-path',
        help='File path to store cookie and serial number in',
        default=Defaults.FILES_PATH,
        dest='file_path')
    parser.add_argument('method', help='HTTP method')

    commands_parser = parser.add_subparsers(
        help='commands',
        dest='command')

    functions_list = inspect.getmembers(Urls, predicate=inspect.ismethod)

    for function in functions_list:
        if not function[0] in _IGNORE_LIST:
            function_parser = commands_parser.add_parser(
                function[0],
                help=function[1].__doc__.strip())

            for item in inspect.signature(function[1]).parameters.items():
                function_parser.add_argument(item[0], help=item[0])

    args = parser.parse_args()

    connector = ApiConnector(args.username, args.password, Defaults.SMART_PHONE_ID, args.file_path)

    try:
        url = _get_url(args)
        result = connector.query(url, args.method)
        print(json.dumps(result, indent=4))
    except ApiError as e:
        print('Error from {}: {}'.format(e.response.url, e.response.text))


def _get_url(args):
    function = getattr(Urls, args.command)
    arg_names = _get_args_name(function)

    arg_values = []
    parsed = vars(args)
    for name in arg_names:
        arg_values.append(parsed[name])

    f_kwargs = dict(zip(arg_names, arg_values))
    return function(**f_kwargs)


def _get_args_name(function):
    names = []
    for item in inspect.signature(function).parameters.items():
        names.append(item[0])
    return names


if __name__ == "__main__":
    main()



from .constants import Constants
from .mode import Mode, HeatingMode, QuickMode, QuickVeto
from .activemode import ActiveMode
from .timeprogram import TimeProgram, TimeProgramDay, TimeProgramDaySetting
from .component import Component
from .boilerstatus import BoilerStatus
from .circulation import Circulation
from .device import Device
from .holidaymode import HolidayMode
from .hotwater import HotWater
from .room import Room
from .zone import Zone
from .system import System
from .mapper import Mapper
